/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * DisplayDriver.cc - Generic display driver superclass
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../config.h"

#include <sys/types.h>
#include <sys/time.h>

#include <String.h>

#include "global.h"
#include "DisplayDriver.h"

#ifdef USE_SVGALIB
#include "SVGA256.h"
#endif
#ifdef USE_XLIB
#include "Xlib.h"
#endif

struct DisplayDrivers DisplayDrivers[] = {
#ifdef USE_XLIB
  {"xlib", (DisplayDriver *(*)(const String &))newXlib},
#endif
#ifdef USE_SVGALIB
  {"svga256", (DisplayDriver *(*)(const String &))newSVGA256},
#endif
  {NULL}
};

void DisplayDriver::clear(void) {
  POFUNC;
}

void DisplayDriver::free(void) {
  POFUNC;
  clear();
}

DisplayDriver::DisplayDriver(void) {
  POFUNC;
  clear();
}

DisplayDriver::~DisplayDriver() {
  POFUNC;
  free();
}

void DisplayDriver::TickAction(struct timeval *when, TerminalDriver &tty) {
  gettimeofday(when, NULL);
  when->tv_usec+=250000;
  when->tv_sec+=when->tv_usec/1000000;
  when->tv_usec%=1000000;

  tickseq++;
  tty.getCharMap().setCursorCycle(tickseq%2);
  tty.getCharMap().setFlashCycle(tickseq%4==0);

  // Update display if necessary to reflect new cursor/flash state
  Draw(tty, FALSE);
}

DisplayDriver *newDisplayDriver(const String &dn, const String &da) return d {
  PGFUNC;
  D("trying to open \"" << dn << ":" <<da <<"\"");
  d=NULL;

  for(int n=0;DisplayDrivers[n].name&&!d;n++)
    if(!fcompare(dn, DisplayDrivers[n].name))
      if(DisplayDrivers[n].newobject) {
	d=(*DisplayDrivers[n].newobject)(da);
      }

  if(!d)
    return;
}

