/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997,98 Peter Corlett. All Rights Reserved.
 *
 * FontEntry.cc: Functions to read and process font data.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>

#include <fstream.h>
#include <iostream.h>

#include "../config.h"
#include "global.h"

#include "FontEntry.h"
#include "RawFont.h"
#include "FontEntry.OSLSet.h"

FontEntryOSLSet fontcache;

BOOL AddFontsFromDir(const String &fdir) {
  PGFUNC;

  DIR *dir;
  struct dirent *de;
  RawFont f;

  if((dir=opendir(fdir))) {
    while((de=readdir(dir))!=NULL) {
      String fname=fdir+"/"+de->d_name;
      D("Trying filename `" << fname << "'");
      if(!f.LoadBin(fname)) {
	// Is a valid font
	D("Adding font \"" << fname << "\" (" << f.getWidth() << 'x' << f.getHeight() << ")");
	FontEntry *fe;
	if(!(fe=new FontEntry))
	  MEMORY_DIE;
	fe->name=fname;
	fe->width=f.getWidth();
	fe->height=f.getHeight();
	fontcache.add(*fe);
      }
    }
    closedir(dir);
    return TRUE;
  } else {
    return FALSE;
  }
}

BOOL AddFontsFromPath(const String &path) {
  PGFUNC;
  String s=path, dir;

  int idx;
  for(;;) {
    idx=s.index(":");
    if(idx>=0) {
      dir=s.before(idx);
      s=s.after(idx);
      AddFontsFromDir(dir);
    } else {
      AddFontsFromDir(s);
      break;
    }
  }
  return TRUE;
}

String FindBestFont(int width, int height) return bestn {
  PGFUNC;
  bestn=NULL;
  int bestw=0, besth=0;

  Pix p=fontcache.first();

  D("Looking for best fit font to " << width << "x" << height);
  while(p!=0) {
    D("Considering " << fontcache(p).width << "x" << fontcache(p).height << " font");
    // First, make sure this font is no bigger than that requested.
    if((fontcache(p).width<=width)&&(fontcache(p).height<=height))
      // Now, does it have a larger area than what we have, if so, use it.
      if((fontcache(p).width*fontcache(p).height>bestw*besth)) {
	bestw=fontcache(p).width;
	besth=fontcache(p).height;
	bestn=fontcache(p).name;
      }
    fontcache.next(p);
  }
  D("Using font " << bestn << ", " << bestw << "x" << besth);
  return bestn;
}

void FontEntry::clear(void) {
  POFUNC;
  height=0;
  width=0;
  name=NULL;
  description=NULL;
}

void FontEntry::free(void) {
  POFUNC;
  clear();
}

#if 0
FontEntry &FontEntry::operator=(const FontEntry &f) {
  POFUNC;

  free();

  if(!&f) { // Assignment from NULL...
    return *this;
  }

  height=f.height;
  width=f.width;
  name=f.name+"-copy";
  for(int n=0;n<MAX_FONT_GLYPHS;n++) {
    if(f.glyph[n]) {
      if(!(glyph[n]=new BIT[width*height]))
	MEMORY_DIE;
      memcpy(glyph[n], f.glyph[n], width*height*sizeof(BIT));
    }
  }
  return *this;
}
#endif
