/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * RastFunc.cc - Rasterises CharCell[]s to various formats
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../config.h"

#include "global.h"
#include "RastFunc.h"

void RenderCharmapLine8(const CharMap &map, const RawFont &font, int Y,
			const int cmap[16], UBYTE *fbuf, int sw) {
  CharCell c;

  int ch=font.getHeight(), cw=font.getWidth();
  int tw=map.getWidth();

  int fg[tw], bg[tw];
  BIT *gptr[tw];

  for(int X=0; X<tw; X++) {
    c=map.getCharCell(Y*tw+X);
    gptr[X]=font.getGlyph(c.getGlyph());
      
    bg[X]=c.getBack();
    if(c.getFlash()&&map.getFlashCycle())
      fg[X]=c.getBack();
    else
      fg[X]=c.getFore();
    if(!map.getCursorCycle()&&map.getCursorVisibility()&&
       (X==map.getCursorX())&&(Y==map.getCursorY())) {
      bg[X]+=8; fg[X]+=8;
    }
    fg[X]=cmap[fg[X]];
    bg[X]=cmap[bg[X]];
  }
    
  for(int y=0; y<ch; y++) {
    UBYTE *fbpos=fbuf+y*sw;
    UBYTE and, xor;
    for(int X=0; X<tw; X++) {
      xor=bg[X]; and=fg[X]^bg[X];
	
      // Duff's device follows:
      int n=(cw+7)/8, c=cw;
      switch(c%8) {
      case 0:
	do {
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 7:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 6:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 5:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 4:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 3:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 2:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 1:
	  *fbpos++=xor^((*gptr[X]++)&and);
	}while(--n>0);
      }
    }
  }
}

void RenderCharMap8(const CharMap &map, const CharMap &map2,
		    const RawFont &font, const int cmap[16],
		    UBYTE *fbuf, int sw) {
  int th=map.getHeight(), ch=font.getHeight();
  for(int Y=0; Y<th; Y++) {
    if(map.CompareLines(map2, Y)) {
      RenderCharmapLine8(map, font, Y, cmap, fbuf+Y*sw*ch, sw);
    }  
  }
}

void RenderCharmapLine16(const CharMap &map, const RawFont &font, int Y,
			 const int cmap[16], UWORD *fbuf, int sw) {
  CharCell c;

  int ch=font.getHeight(), cw=font.getWidth();
  int tw=map.getWidth();

  int fg[tw], bg[tw];
  BIT *gptr[tw];

  for(int X=0; X<tw; X++) {
    c=map.getCharCell(Y*tw+X);
    gptr[X]=font.getGlyph(c.getGlyph());
      
    bg[X]=c.getBack();
    if(c.getFlash()&&map.getFlashCycle())
      fg[X]=c.getBack();
    else
      fg[X]=c.getFore();
    if(!map.getCursorCycle()&&map.getCursorVisibility()&&
       (X==map.getCursorX())&&(Y==map.getCursorY())) {
      bg[X]+=8; fg[X]+=8;
    }
    fg[X]=cmap[fg[X]];
    bg[X]=cmap[bg[X]];
  }
    
  for(int y=0; y<ch; y++) {
    UWORD *fbpos=fbuf+y*sw;
    UWORD and, xor;
    for(int X=0; X<tw; X++) {
      xor=bg[X]; and=fg[X]^bg[X];
	
      // Duff's device follows:
      int n=(cw+7)/8, c=cw;
      switch(c%8) {
      case 0:
	do {
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 7:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 6:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 5:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 4:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 3:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 2:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 1:
	  *fbpos++=xor^((*gptr[X]++)&and);
	}while(--n>0);
      }
    }
  }
}

void RenderCharMap16(const CharMap &map, const CharMap &map2,
		     const RawFont &font, const int cmap[16],
		     UWORD *fbuf, int sw) {
  int th=map.getHeight(), ch=font.getHeight();
  for(int Y=0; Y<th; Y++) {
    if(map.CompareLines(map2, Y)) {
      RenderCharmapLine16(map, font, Y, cmap, fbuf+Y*sw*ch, sw);
    }  
  }
}

void RenderCharmapLine32(const CharMap &map, const RawFont &font, int Y,
			 const int cmap[16], ULONG *fbuf, int sw) {
  CharCell c;

  int ch=font.getHeight(), cw=font.getWidth();
  int tw=map.getWidth();

  int fg[tw], bg[tw];
  BIT *gptr[tw];

  for(int X=0; X<tw; X++) {
    c=map.getCharCell(Y*tw+X);
    gptr[X]=font.getGlyph(c.getGlyph());
      
    bg[X]=c.getBack();
    if(c.getFlash()&&map.getFlashCycle())
      fg[X]=c.getBack();
    else
      fg[X]=c.getFore();
    if(!map.getCursorCycle()&&map.getCursorVisibility()&&
       (X==map.getCursorX())&&(Y==map.getCursorY())) {
      bg[X]+=8; fg[X]+=8;
    }
    fg[X]=cmap[fg[X]];
    bg[X]=cmap[bg[X]];
  }
    
  for(int y=0; y<ch; y++) {
    ULONG *fbpos=fbuf+y*sw;
    ULONG and, xor;
    for(int X=0; X<tw; X++) {
      xor=bg[X]; and=fg[X]^bg[X];
	
      // Duff's device follows:
      int n=(cw+7)/8, c=cw;
      switch(c%8) {
      case 0:
	do {
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 7:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 6:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 5:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 4:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 3:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 2:
	  *fbpos++=xor^((*gptr[X]++)&and);
	case 1:
	  *fbpos++=xor^((*gptr[X]++)&and);
	}while(--n>0);
      }
    }
  }
}

void RenderCharMap32(const CharMap &map, const CharMap &map2,
		     const RawFont &font, const int cmap[16],
		     ULONG *fbuf, int sw) {
  int th=map.getHeight(), ch=font.getHeight();
  for(int Y=0; Y<th; Y++) {
    if(map.CompareLines(map2, Y)) {
      RenderCharmapLine32(map, font, Y, cmap, fbuf+Y*sw*ch, sw);
    }  
  }
}

