/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * RawFont.h: Header for RawFont.cc
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef RAWFONT_H
#define RAWFONT_H

#include <iostream.h>
#include <String.h>

#define MAX_FONT_GLYPHS 1024

/// Contains a raw fixed-width font.
class RawFont {
  /// Height of glyphs
  int height;
  /// Width of glyphs
  int width;
  /// Name of font where appropriate
  String name;
  /// Array of pointers to glyph data
  BIT *glyph[MAX_FONT_GLYPHS];
  ///
  void clear(void);
  ///
  void free(void);
  ///
  friend BOOL AddRawFontsFromDir(const String &s);

  ///
public:
  /// Default constructor.
  RawFont(void) { POFUNC; clear(); };
  /// Copy constructor.
  RawFont(const RawFont &f) {POFUNC; clear(); *this=f;};
  /// Destructor.
  ~RawFont() { POFUNC; free(); };

  /// Assignment.
  RawFont &operator=(const RawFont &f);

  /// Returns nonzero is object is valid.
  operator void*(void) const {
    return width*height?(void *)TRUE:NULL;
  }
    
  /// Returns nonzero if object is invalid.
  BOOL operator!(void) const {
    return (*this)?FALSE:TRUE;
  };

  /// Returns the width of the character glyphs.
  int getWidth(void) const { return width; };
  /// Returns the height of the character glyphs.
  int getHeight(void) const { return height; };
  /// Returns a pointer to the font data for glyph `n'.
  BIT *getGlyph(int n) const { return glyph[n]?:glyph[0]; };

  /// (only used by GenFont)
  int LoadPBM(const String &);

  /// Loads the font data from a binary file `s'
  int LoadBin(const String &s);
  /// Saves the font data to a binary file `s'.
  int SaveBin(const String &s, int first=0, int last=MAX_FONT_GLYPHS) const;
  ///
  BOOL MakeViewdataGlyphs(void);

};

#endif /* RAWFONT_H */
