/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * global.h: Global variables, macros, and configuration data.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef GLOBAL_H
#define GLOBAL_H

#define PROGNAME "XViewdata"
#define VERSION "1.0.3"

/* Bits and pieces I use from Amiga programming */
typedef unsigned char UBYTE;
typedef signed char BYTE;
typedef unsigned short UWORD;
typedef signed short WORD;
typedef unsigned int ULONG;
typedef signed int LONG;
typedef int BOOL;
#define TRUE (~0)
#define FALSE 0

typedef signed char BIT;

/* General panic function */
#define DIE(n) {cerr << PROGNAME << ": " << n << "\n(in " << __FILE__ << ":" << __LINE__ \
  << " " << __PRETTY_FUNCTION__ << ")\n"; exit(1);}

/* Special case of die() where we ran out of core */
#define MEMORY_DIE DIE("No free memory left")


/* Small hackette for debugging */
/* Use -DDEBUG when compiling to enable this debugging macro. There will be
 * lots of debugging output...
 */

#ifndef DEBUG
#define D(s)
#define PGFUNC
#define POFUNC
#else
#include <iostream.h>
#define D(s) { \
		 cerr << "\33[34m" __FILE__ << ":" << __LINE__ \
		      << ": \33[32m" << __PRETTY_FUNCTION__ << "\n \33[35m" \
		      << s << "\33[0m\n";}
#define PGFUNC { \
  cerr << "\33[34m" << __FILE__ << ":" << __LINE__ \
		      << ": \33[1;31m" << __PRETTY_FUNCTION__ << "\33[0m\n";}
#define POFUNC { \
		 cerr << "\33[34m" << __FILE__ << ":" << __LINE__ \
		      << ": \33[1;31m" << __PRETTY_FUNCTION__ \
		      << "  \33[0;35m" << (unsigned int) this << "\33[0m\n";}
#endif
  
#endif /* GLOBAL_H */
